#//-----------------------------------------------
#//TUSB-K01DAZ用ドライバアクセス用API
#//Python 用モジュール
#//
#//2024_04_26　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

from ctypes import *

#Device DLL Loader ( cdecl )
tusbk01dadll = cdll.LoadLibrary("TUSBKDA.DLL")

#######  dll function definitions (do not call directry from user codes)
tusbk01dadll.TUSBK01DA_Device_Open.restype = c_short
tusbk01dadll.TUSBK01DA_Device_Open.argtypes = (c_short,)
tusbk01dadll.TUSBK01DA_Device_Close.restype = None
tusbk01dadll.TUSBK01DA_Device_Close.argtypes = (c_short,)
tusbk01dadll.TUSBK01DA_DacOut.restype = c_short
tusbk01dadll.TUSBK01DA_DacOut.argtypes = (c_short,c_short)
tusbk01dadll.TUSBK01DA_IntClk.restype = c_double
tusbk01dadll.TUSBK01DA_IntClk.argtypes = (c_int,)
tusbk01dadll.TUSBK01DA_Da_Start.restype = c_short
tusbk01dadll.TUSBK01DA_Da_Start.argtypes = (c_short,c_int,c_short)
tusbk01dadll.TUSBK01DA_Da_Stop.restype = c_short
tusbk01dadll.TUSBK01DA_Da_Stop.argtypes = (c_short,)
tusbk01dadll.TUSBK01DA_Da_Status.restype = c_short
tusbk01dadll.TUSBK01DA_Da_Status.argtypes = (c_short,POINTER(c_ubyte))
tusbk01dadll.TUSBK01DA_Data_Set.restype = c_short
tusbk01dadll.TUSBK01DA_Data_Set.argtypes = (c_short,POINTER(c_short),c_short)
tusbk01dadll.TUSBK01DA_ExtTimer.restype = c_double
tusbk01dadll.TUSBK01DA_ExtTimer.argtypes = (c_int,)
tusbk01dadll.TUSBK01DA_ExtTimer_Set.restype = c_short
tusbk01dadll.TUSBK01DA_ExtTimer_Set.argtypes = (c_short,c_int)
tusbk01dadll.TUSBK01DA_DIO_Dir.restype = c_short
tusbk01dadll.TUSBK01DA_DIO_Dir.argtypes = (c_short,c_ubyte,c_ubyte)
tusbk01dadll.TUSBK01DA_DIO_Out.restype = c_short
tusbk01dadll.TUSBK01DA_DIO_Out.argtypes = (c_short,c_ubyte,c_ubyte)
tusbk01dadll.TUSBK01DA_DIO_In.restype = c_short
tusbk01dadll.TUSBK01DA_DIO_In.argtypes = (c_short,POINTER(c_ubyte))
tusbk01dadll.TUSBK01DA_DIO_Status.restype = c_short
tusbk01dadll.TUSBK01DA_DIO_Status.argtypes = (c_short,POINTER(c_ubyte))

#######  support functions (do not call directry from user codes)
def isint(n):
    return type(n) is int

def IsInRange(Val,Min,Max):
    if(isint(Val) == False ):
        return False
    elif(Val < Min):
        return False
    elif(Val > Max):
        return False
    else:
        return True

def IsValidId(DevId):
    return IsInRange(DevId,0,15)

# def functions

def TUSBK01DA_Device_Open(DevId):
    """Open Device
    
    Arguments:
    DevId           ----    Device Id 0~15
    
    Returns:
    int             ----    Result code
    """
    if( IsValidId(DevId) ):
        return tusbk01dadll.TUSBK01DA_Device_Open(DevId)
    else:
        return 8    #Invalid parameter

def TUSBK01DA_Device_Close(DevId):
    """Close Device

    Arguments:
    DevId           ----    Device Id 0~15
    """
    if( IsValidId(DevId) ):
        tusbk01dadll.TUSBK01DA_Device_Close(DevId)
    else:
        return 8    #Invalid parameter

def TUSBK01DA_DacOut(DevId,Data):
    """Set Data to DAC
    
    Arguments:
    DevId           ----    Device Id 0~15
    Data            ----    Output Data -8192~8192

    Returns:
    int             ----    Result code
    """
    if( IsValidId(DevId) ):
        tusbk01dadll.TUSBK01DA_DacOut(DevId,Data)
        return 0
    else:
        return 8    #Invalid parameter
    
def TUSBK01DA_IntClk(Clk):
    """Calculate timer period from interval setting number

    Arguments:
    Clk             ----    Clock Interval Number 50,000~1,000,000,000 [ns]

    Returns:
    double          ----    calculated period or 0.0(invalid Clk range)
    """
    if( IsInRange(Clk,50000,1000000000)):
        return tusbk01dadll.TUSBK01DA_IntClk(Clk)
    else:
        return 0.0

def TUSBK01DA_Da_Start(DevId,Clk,dLen):
    """Start Continuous DA conversion
    
    Arguments:
    DevId           ----    Device Id 0~15
    Clk             ----    Clock Interval Number 50,000~1,000,000,000 [ns]
                            The external Clock selected if Clk is 0 
    dLen            ----    Wave Image Data length

    Returns:
    int             ----    Result code
    """
    if not IsValidId(DevId): return 8
    if not ( IsInRange(Clk,50000,1000000000) or IsInRange(Clk,0,0)): return 8
    if not IsInRange(dLen,1,4096): return 8
    return tusbk01dadll.TUSBK01DA_Da_Start(DevId,Clk,dLen)

def TUSBK01DA_Da_Stop(DevId):
    """Stop Continuous DA conversion

    Arguments:
    DevId           ----    Device Id 0~15

    Returns:
    int             ----    Result code
    """
    if IsValidId(DevId):
        return tusbk01dadll.TUSBK01DA_Da_Stop(DevId)
    else:
        return 8    #Invalid parameter
    
def TUSBK01DA_Da_Status(DevId):
    """Get DA operating condition
    
    Arguments:
    DevId           ----    Device Id 0~15

    Returns:
    int,            ----    Result code
    int,            ----    Status 0:DC Mode 1:Waveform generating
    """
    if IsValidId(DevId):
        Status = c_ubyte()
        ret = tusbk01dadll.TUSBK01DA_Da_Status(DevId,byref(Status))
        if ret == 0:
            return 0,Status.value
        else:
            return ret,None
    else:
        return 8,None

def TUSBK01DA_Data_Set(DevId,Data,Leng):
    """Write Wave Image Data Sets
    
    Arguments:
    DevId           ----    Device Id 0~15
    Data            ----    Wave Image Data Tuple (or List)
    Leng            ----    Wave Image Length

    Returns:
    int             ----    Result code
    """
    if not IsValidId(DevId): return 8
    if not IsInRange(Leng,1,4096): return 8
    if Leng > len(Data): return 8
    c_data = (c_short * Leng)(*Data)
    p_data = cast(pointer(c_data),POINTER(c_short))
    return tusbk01dadll.TUSBK01DA_Data_Set(DevId,p_data,Leng)

def TUSBK01DA_ExtTimer(Clk):
    """Calculate external output timer period from setting number

    Arguments:
    Clk             ----    Clock Interval Number 1,000~500,000,000 [ns]

    Returns:
    double          ----    calculated period or 0.0(invalid Clk range)
    """
    if( IsInRange(Clk,1000,500000000)):
        return tusbk01dadll.TUSBK01DA_ExtTimer(Clk)
    else:
        return 0.0

def TUSBK01DA_ExtTimer_Set(DevId,Clk):
    """Set external output timer
    
    Arguments:
    DevId           ----    Device Id 0~15
    Clk             ----    Clock Interval Number 1,000~500,000,000 [ns]

    Returns:
    int             ----    Result code
    """
    if not IsValidId(DevId): return 8
    if not IsInRange(Clk,1000,500000000): return 8
    return tusbk01dadll.TUSBK01DA_ExtTimer_Set(DevId,Clk)

def TUSBK01DA_DIO_Dir(DevId,bit,dir):
    """Set DIO bit Direction
    
    Arguments:
    DevId           ----    Device Id 0~15
    bit             ----    target bit select 0:bit0 1:bit1
    dir             ----    Direction 0:input 1:output

    Returns:
    int             ----    Result code
    """
    if not IsValidId(DevId): return 8
    if not IsInRange(bit,0,1): return 8
    if not IsInRange(dir,0,1): return 8
    return tusbk01dadll.TUSBK01DA_DIO_Dir(DevId,bit,dir)

def TUSBK01DA_DIO_Out(DevId,bit,data):
    """Set DIO bit Output Data
    
    Arguments:
    DevId           ----    Device Id 0~15
    bit             ----    target bit select 0:bit0 1:bit1
    data            ----    output logic 0:low 1:high

    Returns:
    int             ----    Result code
    """
    if not IsValidId(DevId): return 8
    if not IsInRange(bit,0,1): return 8
    if not IsInRange(data,0,1): return 8
    return tusbk01dadll.TUSBK01DA_DIO_Out(DevId,bit,data)

def TUSBK01DA_DIO_In(DevId):
    """Read DIO Input Data
    
    Arguments:
    DevId           ----    Device Id 0~15

    Returns:
    int,            ----    Result code
    int             ----    Input bit pattern
    """
    if IsValidId(DevId):
        data = c_ubyte()
        ret = tusbk01dadll.TUSBK01DA_DIO_In(DevId,byref(data))
        if ret == 0:
            return 0,data.value
        else:
            return ret,None
    else:
        return 8,None
    
def TUSBK01DA_DIO_Status(DevId):
    """Read DIO Port Settings
    
    Arguments:
    DevId           ----    Device Id 0~15

    Returns:
    int,            ----    Result code
    int,            ----    bit0 Status 0:input 1:Output Low 2:Output High
    int,            ----    bit1 Status 0:input 1:Output Low 2:Output High
    """
    if IsValidId(DevId):
        c_data = (c_ubyte * 2)()
        p_data = cast(pointer(c_data),POINTER(c_ubyte))
        ret = tusbk01dadll.TUSBK01DA_DIO_Status(DevId,p_data)
        if ret == 0:
            return 0,p_data[0],p_data[1]
        else:
            return ret,None,None
    else:
        return 8,None,None

# Error Messages from return coddes
ErrStrDic = {
            0:"正常終了しました",
            2:"ドライバが初期化されていません",
            3:"すでにデバイスはオープンされています",
            4:"接続されている台数が多すぎます",
            5:"オープンできませんでした",
            6:"デバイスがみつかりません",
            8:"パラメータエラーです",
            9:"USB通信エラーです",
            99:"その他のエラーです"
}
def GetErrMessage(retcode):
    if( retcode in ErrStrDic):
        return ErrStrDic[retcode]
    else:
        return "不明なエラーです"

