#//-----------------------------------------------
#//TUSB-K01DAZ用サンプルプログラム
#//Python 用モジュール
#//
#//2024_04_26　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------
import TUSBKDA
from SpFuncs import *
import sys
import math

class Tusbk01da_Device:
    """TUSB-K01DAZ デバイス操作クラス フォーム"""

    DevId = None    # デバイスID
    mon_t = None    # DIOモニタータイマー

    def __init__(self,DevId):
        """イニシャライザ
        
         Arguments:
         DevId           ----    Device id 0~15
         """
        ret = TUSBKDA.TUSBK01DA_Device_Open(DevId)
        if ret == 0:    # 正常にオープン出来た場合
            self.DevId = DevId                                      # デバイスIDをクラスメンバ変数に保持する
            self.form.protocol("WM_DELETE_WINDOW",self.DevClose)
            if self.DevInit():                                      # デバイスの初期化処理
                self.mon_t = intTT(0.5,self.Monitor)                    # モニタータイマー作成、コールバック関数登録
                self.mon_t.Start()                                      # モニタータイマースレッド開始
                self.WidgetState("Stop")
            else:
                TUSBKDA.TUSBK01DA_Device_Close(DevId)                 # デバイス初期化に失敗したらデバイスを閉じる
                self.form.destroy()                                     # フォームも閉じる                
        else:
            ShowInfo(TUSBKDA.GetErrMessage(ret))                  # 失敗エラーメッセージ表示
            self.form.destroy()                                     # フォームを閉じる

    def DevInit(self):
        """デバイスの状態を確認して画面表示に反映する"""
        ret = TUSBKDA.TUSBK01DA_DIO_Status(self.DevId)
        if ret[0] != 0:
            ShowInfo(TUSBKDA.GetErrMessage(ret[0]))
            return False
        self.cmbb_dio[0].current(ret[1])    # DIO現在状態を表示に反映する
        self.cmbb_dio[1].current(ret[2])
        # 以下ボタンやコンボボックス等のイベントハンドラを登録する処理
        self.cmbb_dio[0].bind("<<ComboboxSelected>>", lambda e: self.Dio_Change_on(0))
        self.cmbb_dio[1].bind("<<ComboboxSelected>>", lambda e: self.Dio_Change_on(1))
        self.txt_extclk.bind("<KeyRelease>",lambda e: self.ExClk_Change_on())
        self.btn_exclk.config(command=lambda: self.Clock_out_on())
        self.btn_da.config(command=lambda: self.Convert_on())
        self.rbtn_wclksel[0].config(command=lambda: self.Clk_ExIn_on())
        self.rbtn_wclksel[1].config(command=lambda: self.Clk_ExIn_on())
        self.txt_wclkdiv.bind("<KeyRelease>",lambda e: self.Request_Clock_Change_on())
        self.btn_start.config(command=lambda: self.Start_on())
        self.btn_stop.config(command=lambda: self.Stop_on())

        self.ExClk_Change_on()
        self.Request_Clock_Change_on()
        return True
    
    def DevClose(self):
        """デバイスを閉じる時の処理"""
        if self.DevId != None:
            self.mon_t.Stop()                               # モニタタイマー終了
            TUSBKDA.TUSBK01DA_Device_Close(self.DevId)    # デバイスクローズ
            self.form.destroy()                             # フォームを閉じる
            self.DevId = None

    def WidgetState(self,stat):
        """状態に応じてウィジットの表示を切り替える処理
        
        Arguments
        stat            ----    設定状態を表す文字列

        'Stop' - デバイスが接続されていて連続取り込み実行していない時
        'Run' - デバイスが接続されていて連続取り込みを実行している時
        """
        EnStop = tk.NORMAL if( stat == 'Stop' ) else tk.DISABLED
        EnRun = tk.NORMAL if( stat == 'Run' ) else tk.DISABLED
        self.btn_da['state'] = EnStop
        self.btn_start['state'] = EnStop
        self.btn_stop['state'] = EnRun
        self.IntClockWidgetState()

    def IntClockWidgetState(self):
        """内部クロック設定ウィジットの表示を切り替える処理"""
        if self.ClkSelV.get() == 0: # External selected
            self.txt_wclkdiv['state'] = tk.DISABLED
        else: # Internal selected
            self.txt_wclkdiv['state'] = tk.NORMAL

    # timer -------------------------------------
    def Monitor(self):
        """モニタータイマー DIO入力チェック"""
        ret = TUSBKDA.TUSBK01DA_DIO_In(self.DevId)                        # ディジタル入力を読み込む
        if ret[0] == 0:
            self.lbl_dio[0]['text'] = "High" if bitChk(ret[1],0) else "Low"
            self.lbl_dio[1]['text'] = "High" if bitChk(ret[1],1) else "Low"

    # 補助的関数/////////////////////////
    def Dio_Set(self,Bit,Dir,Dat):
        """DIOの設定関数
        
        Arguments:
        Bit             ----    設定ビット 0 or 1
        Dir             ----    入出力方向 0(IN) or 1(OUT)
        Dat             ----    出力データ 0(Low) or 1(High)
        """
        if Dir == 1:    # 出力であれば
            ret = TUSBKDA.TUSBK01DA_DIO_Out(self.DevId,Bit,Dat)   # ディジタル出力値を設定
            if ret != 0:
                ShowInfo(TUSBKDA.GetErrMessage(ret))
                return
        ret = TUSBKDA.TUSBK01DA_DIO_Dir(self.DevId,Bit,Dir)       # 入出力方向を設定
        if ret != 0:
            ShowInfo(TUSBKDA.GetErrMessage(ret))

    def VoltToDData(self,volt):
        """電圧値からDAデータに変換する。
        範囲外の場合は勝手に範囲に収める"""
        if volt > 2.5: return 8191
        if volt < -2.5: return -8192
        data = int(round( volt / 2.5 * 8192.0))
        if data > 8191: return 8191
        if data < -8192: return -8192
        return data
    
    def RectangleF(self,angle):
        angle /= (2.0 * math.pi)    # 2Pi->1.0　スケール変換
        angle %= 1                  # 小数点以下のみ切り出し(周期関数)
        if angle < 0.5: return 1.0
        else: return -1.0

    def TriangleF(self,angle):
        """三角波生成"""
        angle /= (2.0 * math.pi)    # 2Pi->1.0　スケール変換
        angle %= 1                  # 小数点以下のみ切り出し(周期関数)
        angle *= 4.0                # 1.0->4.0 スケール変換
        if angle < 1.0: return angle
        elif angle < 3.0: return 2.0-angle
        else: return angle-4.0

    def GenWaveData(self,shape,len,ampl,offset):
        """波形データを生成する
        
        arguments:
        shape           ----    波形 0:sin 1:三角 2:矩形
        len             ----    データ長
        ampl            ----    振幅 (peak to peakの半分)
        offset          ----    波形の中間の電圧
        
        return:
        [int,...]       ----    データリスト
        """
        retdata = list()
        for cnt in range(len):
            angle = 2.0 * math.pi * float(cnt) / float(len)
            if shape == 0: #sin
                retdata.append(self.VoltToDData(ampl*math.sin(angle)+offset))
            elif shape == 1: #triangle
                retdata.append(self.VoltToDData(ampl*self.TriangleF(angle)+offset))
            elif shape == 2: #rectangle
                retdata.append(self.VoltToDData(ampl*self.RectangleF(angle)+offset))
            else:
                retdata.append(0)
        return retdata

    #/// ボタン等ウィジットの操作イベントハンドラ /////////////////////////
    def Dio_Change_on(self,Bit):
        """DIOの設定が変更された"""
        ststr = self.cmbb_dio[Bit].get()
        if ststr == 'INPUT':
            self.Dio_Set(Bit,0,0)
        elif ststr == 'LOW':
            self.Dio_Set(Bit,1,0)
        elif ststr == 'HIGH':
            self.Dio_Set(Bit,1,1)
        elif ststr == 'CLOCK':
            """ディジタルIOの設定ではクロックは選択しない。外部クロックで
            出力設定をすると自動的にクロック出力になる。手動でクロックが
            選択された時は、現在のDIOの状態を確認してそれを出力状態
            コンボボックスに設定しなおす"""
            if Bit == 1:    # クロックがあるのはDIO1のみ
                ret = TUSBKDA.TUSBK01DA_DIO_Status(self.DevId)      # 現在の出力状態の確認
                if ret[0] != 0:
                    ShowInfo(TUSBKDA.GetErrMessage(ret[0]))
                    return False
                if ret[2] != 3:                                     # クロック設定以外の時はコンボボックスの選択をそれに合わせる
                    self.cmbb_dio[Bit].current(ret[2])

    def ExClk_Change_on(self):
        """外部出力クロックの設定値が変更された"""
        Clk = ForceToInt(self.txt_extclk.get())
        ret = TUSBKDA.TUSBK01DA_ExtTimer(Clk)                 # 外部クロックに設定されうる値を取得する
        if ret == 0.0:
            self.lbl_exclk['text'] = "Invalid Divide Setting"
        else:
            self.lbl_exclk['text'] = str(ret)

    def Clk_ExIn_on(self):
        """内部クロック、外部クロックの選択が変更された"""
        self.IntClockWidgetState()

    def Clock_out_on(self):
        """クロック出力設定"""
        Clk = ForceToInt(self.txt_extclk.get())
        ret = TUSBKDA.TUSBK01DA_ExtTimer_Set(self.DevId,Clk)  # 外部クロック出力設定
        if ret != 0:
            ShowInfo(TUSBKDA.GetErrMessage(ret))
            return False
        else:
            self.cmbb_dio[1].current(3)

    def Convert_on(self):
        """DA単一変換実行"""
        volt = ForceToFloat(self.txt_da.get(),err = -10.0)
        if volt == -10.0:   # error
            ShowInfo("Invalid voltage setting")
            return
        dacdata = self.VoltToDData(volt)
        ret = TUSBKDA.TUSBK01DA_DacOut(self.DevId,dacdata)
        if ret != 0:
            ShowInfo(TUSBKDA.GetErrMessage(ret))

    def Request_Clock_Change_on(self):
        """内部クロック設定値が変更された"""
        Clk = ForceToInt(self.txt_wclkdiv.get())
        ret = TUSBKDA.TUSBK01DA_IntClk(Clk)                   # 内部クロックに設定されうる値を取得する
        if ret == 0.0:
            self.lbl_wclkdiv['text'] = "Invalid Clock Setting"
        else:
            self.lbl_wclkdiv['text'] = format(ret,".0f")

    def Start_on(self):
        """DA連続出力開始"""
        shape = self.cmbb_wshape.current()
        len = ForceToInt(self.txt_wlen.get())
        if len < 1 or len > 4096:
            ShowInfo("DataLength設定に誤りがあります")
            return
        ampl = ForceToFloat(self.txt_wampl.get(),-10.0)
        if ampl < 0.0 or ampl > 5.0:
            ShowInfo("Amplitude設定に誤りがあります")
            return
        offset = ForceToFloat(self.txt_wofs.get(),-10.0)
        if offset < -2.5 or offset > 2.5:
            ShowInfo("Offset設定に誤りがあります")
            return
        if self.ClkSelV.get() == 0: # external clock
            clkdiv = 0
        else:
            clkdiv = ForceToInt(self.txt_wclkdiv.get())
        wavdata = self.GenWaveData(shape,len,ampl/2.0,offset)
        ret = TUSBKDA.TUSBK01DA_Data_Set(self.DevId,wavdata,len)
        if ret != 0:
            ShowInfo(TUSBKDA.GetErrMessage(ret))
            return
        ret = TUSBKDA.TUSBK01DA_Da_Start(self.DevId,clkdiv,len)
        if ret != 0:
            ShowInfo(TUSBKDA.GetErrMessage(ret))
            return
        self.WidgetState("Run")                     # フォームウィジェット表示をRun状態にする

    def Stop_on(self):
        """AD連続出力停止"""
        ret = TUSBKDA.TUSBK01DA_Da_Stop(self.DevId)
        if ret != 0:
            ShowInfo(TUSBKDA.GetErrMessage(ret))
        self.WidgetState("Stop")                        # フォームウィジェット表示をStop状態にする

    # フォームとウィジェットを生成する処理---------------------------
    form = tk.Tk()
    form.geometry('448x495')
    form.title('TUSB-K01DAZ Sample Program')

    frame = GenList(4,None)     # フレーム(枠)

    # DIOフレーム
    DioSettDictCh1= {
        "INPUT":0,
        "LOW":1,
        "HIGH":2,
    }
    DioSettDictCh2= {
        "INPUT":0,
        "LOW":1,
        "HIGH":2,
        "CLOCK":3,
    }
    lbl_dio = GenList(2,None)   # DIO状態表示ラベル
    cmbb_dio = GenList(2,None)  # DIO設定コンボボックス
    frame[0] = GetLFrame(form,'DIO',0.95,0.12,0.025,0.02)
    GetLabel(frame[0],"D0",0.05,0.5)
    GetLabel(frame[0],"D1",0.55,0.5)
    lbl_dio[0] = GetLabelDisp(frame[0],"",0.15,0.15,0.5)
    lbl_dio[1] = GetLabelDisp(frame[0],"",0.15,0.65,0.5)
    cmbb_dio[0] = GetComboBox(frame[0],list(DioSettDictCh1.keys()),0.15,0.33,0.5)
    cmbb_dio[1] = GetComboBox(frame[0],list(DioSettDictCh2.keys()),0.15,0.83,0.5)

    # Clock Output フレーム
    txt_exclk = None            # クロック出力設定入力
    lbl_exclk = None            # クロック出力設定値表示ラベル
    btn_exclk = None            # クロック出力ボタン
    frame[1] = GetLFrame(form,'Clock Output',0.95,0.14,0.025,0.14)
    GetLabel(frame[1],"Request",0.4,0.2)
    GetLabel(frame[1],"ns",0.9,0.2)
    GetLabel(frame[1],"Actual",0.4,0.7)
    GetLabel(frame[1],"ns",0.9,0.7)
    txt_extclk = GetEntry(frame[1],"1000",0.33,0.55,0.2)
    lbl_exclk = GetLabelDisp(frame[1],"",0.33,0.55,0.7)
    btn_exclk = GetButton(frame[1],"Clock Out",None,0.2,0.12,0.45)

    # DAC Set フレーム
    txt_da = None               # DA変換値入力
    btn_da = None               # シングルDA変換ボタン
    frame[2] = GetLFrame(form,'DAC Set',0.95,0.14,0.025,0.28)
    GetLabel(frame[2],"[V]    -2.5~2.5[V]",0.6,0.5)
    txt_da = GetEntry(frame[2],"0",0.2,0.35,0.5)
    btn_da = GetButton(frame[2],"Convert",None,0.2,0.12,0.5)

    # Wave Generator フレーム
    WaveShapeDict= {
        "Sin":0,
        "Rectangle":1,
        "Square":2,
    }
    ClockSelDict={
        "External":0,
        "Internal":1,
    }
    cmbb_wshape = None              # 波形設定コンボボックス
    txt_wampl = None                # 振幅入力
    txt_wofs = None                 # オフセット入力
    txt_wlen = None                 # データ長入力
    rbtn_wclksel = GenList(2,None)  # クロックソース選択ラジオボタン
    ClkSelV = tk.IntVar(value=1)    # ラジオボタンの選択
    txt_wclkdiv = None              # クロック周期入力
    lbl_wclkdiv = None              # クロック周期設定値テキスト
    btn_start = None                # Start ボタン
    btn_stop = None                 # Stop ボタン
    frame[3] = GetLFrame(form,'Wave Generator',0.95,0.56,0.025,0.42)
    ry = 0.062                      # 高さ位置調整用変数
    ypitch = 0.142                  # このフレーム各行の高さピッチ
    GetLabel(frame[3],"Shape",0.03,ry)
    cmbb_wshape = GetComboBox(frame[3],list(WaveShapeDict.keys()),0.25,0.25,ry)
    ry += ypitch
    GetLabel(frame[3],"Amplitude(p-p)",0.03,ry)
    txt_wampl = GetEntry(frame[3],"5",0.2,0.25,ry)
    GetLabel(frame[3],"[V]",0.45,ry)
    ry += ypitch
    GetLabel(frame[3],"Offset",0.03,ry)
    txt_wofs = GetEntry(frame[3],"0",0.2,0.25,ry)
    ry += ypitch
    GetLabel(frame[3],"Datalength",0.03,ry)
    txt_wlen = GetEntry(frame[3],"1000",0.2,0.25,ry)
    GetLabel(frame[3],"1~4096",0.45,ry)
    ry += ypitch
    GetLabel(frame[3],"Clock",0.03,ry)
    rbtn_wclksel = GetRadioButtons(frame[3],ClockSelDict,0.12,ry,0.17,0,ClkSelV)
    GetLabel(frame[3],"Request",0.5,ry)
    txt_wclkdiv = GetEntry(frame[3],"100000",0.25,0.65,ry)
    GetLabel(frame[3],"ns",0.9,ry)
    ry += ypitch
    GetLabel(frame[3],"Actual",0.5,ry)
    lbl_wclkdiv = GetLabelDisp(frame[3],"",0.25,0.65,ry)
    GetLabel(frame[3],"ns",0.9,ry)
    ry += ypitch
    btn_start = GetButton(frame[3],"Start",Start_on,0.2,0.12,ry)
    btn_stop = GetButton(frame[3],"Stop",Stop_on,0.2,0.42,ry)

# 実行部 ---------------------------------
if __name__=="__main__":
    DevId = 0
    if( len(sys.argv) >= 2 ):
        if sys.argv[1].isdigit():
            DevId = int(sys.argv[1])    # コマンドライン引数でID設定
    tmp = Tusbk01da_Device(DevId=DevId)
    tmp.form.mainloop()

